# -*- coding: utf-8 -*-
"""
Добавление рамки для ленты времени к окну с картинкой перекрёстка 
Добавление соответствующего пункта контекстного меню для светофоров, районов и маршрутов
(c)2013 ЗАО "РИПАС"
www.ripas.ru
"""

from gui import mainWindow
from RoadCentre import kernel, plugins
from PythonQt import QtCore, QtGui
import RoadCentre
import sys


def moveRouteToLayer(route):
    layer = kernel.graphicsScene.addLayer(u'Маршруты')
    layer.addItem(route.graphicsItem())


def getRouteName(dirname, route):
    import os
    return os.path.join(dirname, u'%d%s.xr' % (route.id, route.name))


def selectionChanged():
    sel = kernel.selection()
    timelineFrame = mainWindow.findChild(QtGui.QFrame, 'timelineFrame')
    if timelineFrame == None:
        return
    timelineFrame.visible = False
    if not len(sel):
        return
    # timeline=sel[0].findChild('TimelineObject','timeline')
    timeline = sel[0].property('timeline')
    if timeline == None:
        return
    import settings
    attached_deepness_sec = int(settings.value('TimelinePlugin/attached_deepness_sec', 'Plugins', '60'))
    timelineFrame.show()
    timeline.attachToWidget(timelineFrame, attached_deepness_sec)


def actionAddRoute(checked):
    if checked:
        actionAddRoute.currentRoute = plugins.TimelinePlugin.CreateRoute(kernel.graphicsScene, True)
        moveRouteToLayer(actionAddRoute.currentRoute)
        kernel.selectionModel.clearSelection()
        kernel.itemModel.addObjects(u'Маршруты', (actionAddRoute.currentRoute,))
    else:
        actionAddRoute.currentRoute.stopInteractiveCreation(True)
        if actionAddRoute.currentRoute.name:
            actionAddRoute.currentRoute.saveToXml(
                getRouteName(plugins.TimelinePlugin.routesDir, actionAddRoute.currentRoute))
        actionAddRoute.currentRoute = None


actionAddRoute.currentRoute = None


def addTimelineToObjects():
    cross_view = mainWindow.findChild(QtGui.QWidget, 'crossView')
    parent_widget = cross_view.parent()
    if parent_widget.findChild(QtGui.QFrame, 'timelineFrame'):
        return
    layout = parent_widget.layout()

    timelineFrame = QtGui.QFrame(parent_widget)
    timelineFrame.setFrameShape(QtGui.QFrame.NoFrame)
    timelineFrame.setObjectName('timelineFrame')
    timelineFrame.maximumHeight = 25

    layout.addWidget(timelineFrame)
    kernel.selectionModel.connect('selectionChanged(const QItemSelection&, const QItemSelection&)', selectionChanged)


def docTimelineWidget(d):
    if not mainWindow.restoreDockWidget(d):
        i = 0
        while True:
            doc = mainWindow.findChild('QDockWidget', 'TimelineWidget_%d' % i)
            if not doc:
                break
            if doc != d and not doc.isFloating():
                break
            i = i + 1
        if doc:
            mainWindow.tabifyDockWidget(doc, d)
        else:
            d.floating = True
    if sys.platform.startswith("linux"):
        d.setWindowFlag(0x400,False)
    d.show()
    getattr(d, 'raise')()
    import settings
    scale_on_dock_changed = int(settings.value('TimelinePlugin/scale_on_dock_changed', 'Plugins', '1'))
    if scale_on_dock_changed == 1:
        d.connect('dockLocationChanged(Qt::DockWidgetArea)', d.findChild('TimelineWidget'),
                  'on_actionAdjustScale_triggered()')
    else:
        d.findChild('TimelineWidget').on_actionAdjustScale_triggered()


def areaTimeline():
    from RoadCentre import project
    if not project: return

    for a in kernel.selection():
        doc_window = plugins.TimelinePlugin.createTimelineWidget(mainWindow)
        doc_window.setWindowTitle(u'Диаграмма фаз для района %d (%s)' % (a.id, a.name))
        timeline = doc_window.findChild('TimelineWidget')
        timeline.fixTitle()
        distance = 50
        for sel in a.objects():
            timeline.addLight(project.object(sel), distance)
            distance = distance + 50
        timeline.start()
        docTimelineWidget(doc_window)


def controllerTimeline():
    if plugins.TimelinePlugin == None:
        return
    doc_window = plugins.TimelinePlugin.createTimelineWidget(mainWindow)
    timeline = doc_window.findChild('TimelineWidget')
    distance = 50
    for sel in kernel.selection():
        timeline.addLight(sel, distance)
        distance = distance + 50
    findSynchronizerForTimeline(timeline)
    docTimelineWidget(doc_window)


from menu import SubMenu


class RouteMenu(SubMenu):
    def __init__(self):
        SubMenu.__init__(self, (u'Маршрут', u'Маршруты'))
        a = self.menuMain.addAction(u'Скрыть/показать')
        a.connect('triggered()', self.toggleVisible)
        a = self.menuMain.addAction(u'Удалить')
        a.connect('triggered()', self.deleteRoute)
        a = self.menuMain.addAction(u'Лента времени')
        a.connect('triggered()', self.timeline)

    def menu(self, obj):
        m = SubMenu.menu(self, obj)
        if len(obj) > 1:
            return m
        ro = obj[0]
        if hasattr(ro, 'greenStreets') and len(ro.greenStreets) > 0:
            gsm = m.addMenu(u'Зелёная улица')
            gr = QtGui.QActionGroup(gsm)
            gr.connect('triggered(QAction*)', self.setGreenStreet)
            for g in ro.greenStreets:
                a = gsm.addAction(ro.greenStreets[g])
                a.setData((g, ro))
                gr.addAction(a)
            a = gsm.addAction(u'Отменить')
            a.setData(('', ro))
            gr.addAction(a)
        return m

    def timeline(self):
        if not kernel.selection() or not plugins.TimelinePlugin:
            return
        fw_route = None
        bw_route = None
        for route in kernel.selection():
            if route.forward:
                fw_route = route
                if bw_route:
                    break
            else:
                bw_route = route
                if fw_route:
                    break
        doc_window = plugins.TimelinePlugin.createTimelineWidget(mainWindow, fw_route, bw_route)
        tlw = doc_window.findChild('TimelineWidget')
        if hasattr(RoadCentre, 'priority'):
            RoadCentre.priority.addTimelineWidget(tlw)
            tlw.connect('destroyed(QObject *)', RoadCentre.priority.removeTimelineWidget)
        tlw.start()
        # doc_window.findChild('TimelineWidget').startEmulation('/home/andrey/prj/Spyder/out.dat')
        docTimelineWidget(doc_window)

    def setGreenStreet(self, a):
        data = a.data()
        data[1].setGreenStreet(data[0])

    def toggleVisible(self):
        if not kernel.selection() or not plugins.TimelinePlugin:
            return
        for route in kernel.selection():
            route.visible = not route.visible

    def deleteRoute(self):
        if not kernel.selection():
            return
        f = QtCore.QFile()
        from PythonQt import QtGui
        from gui import mainWindow
        if QtGui.QMessageBox.question(mainWindow, u'Вопрос', u'Удалить выбранные маршруты?',
                                      QtGui.QMessageBox.Yes | QtGui.QMessageBox.No) != QtGui.QMessageBox.Yes:
            return
        for route in kernel.selection():
            f.remove(route.file)
            route.deleteLater()


def addObjectTimeline(obj):
    timeline = obj.property('timeline')
    if timeline:
        return
    timeline_object = plugins.TimelinePlugin.createObjectTimeline(obj)
    timeline_object.autoUpdate = True
    obj.setProperty('timeline', timeline_object)


def initTimeline():
    import settings
    from RoadCentre import project
    plugins.TimelinePlugin.defaultDeepness = int(settings.value('TimelinePlugin/default_deepness', 'Plugins', '300'))
    for obj in project.objects():
        if not obj.inherits('Spectr::Light'):
            continue
        addObjectTimeline(obj)

    routeColor = settings.value('TimelinePlugin/routeColor', 'Plugins')
    if routeColor:
        plugins.TimelinePlugin.routeColor = QtGui.QColor(int(routeColor))
    routeSelectedColor = settings.value('TimelinePlugin/routeSelectedColor', 'Plugins')
    if routeSelectedColor:
        plugins.TimelinePlugin.routeSelectedColor = QtGui.QColor(int(routeSelectedColor))

    routeWidth = settings.value('TimelinePlugin/routeWidth', 'Plugins')
    if routeWidth:
        plugins.TimelinePlugin.routeWidth = float(routeWidth)

    kernel.selectionModel.clearSelection()
    import os
    plugins.TimelinePlugin.routesDir = project.settings.value('Plugins/TimelinePlugin/RoutesDir',
                                                              os.path.join(project.baseDir, 'routes'))
    if not os.path.exists(plugins.TimelinePlugin.routesDir):
        os.mkdir(plugins.TimelinePlugin.routesDir)
    for dirpath, dirnames, filenames in os.walk(plugins.TimelinePlugin.routesDir):
        for name in filenames:
            if os.path.splitext(name)[-1] != '.xr': continue
            new_route = plugins.TimelinePlugin.CreateRoute(kernel.graphicsScene, False)
            if not new_route:
                continue
            if not new_route.loadFromXml(os.path.join(dirpath, name), project):
                continue
            moveRouteToLayer(new_route)
            kernel.itemModel.addObjects(u'Маршруты', (new_route,))
    plugins.TimelinePlugin.routesVisible = plugins.TimelinePlugin.routesVisible


def newObjectConnected(obj_id, isConnected):
    from RoadCentre import project
    obj = project.object(obj_id)
    if not obj:
        return

    if obj and not obj.property("temporary"):
        return
    addObjectTimeline(obj)


def findSynchronizerForTimeline(tlw):
    obj = RoadCentre.project.object(0)
    if obj is None:
        tlw.start()
        return
    resp = obj.sendCommand("GET_REFER")

    def resp_finished():
        try:
            if resp.data[2] == "__time_synchronizer__":
                tlw.setSynchronizer(obj)
            tlw.start()
        except:
            pass

    resp.connect(QtCore.SIGNAL('finished()'), resp_finished)
